/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/* Author: Paul Smit                       Date: December  2000               */
/* Version 1.1 and above                                                      */
/* Author: Arjan van Vught                 Date: October   2001               */
/*                                                                            */
/******************************************************************************/
/* Version 1.0   First release NT                                             */
/* Version 1.1   First release AIX                                            */
/* Version 1.2   First release Sun Solaris                                    */
/* Version 1.3   First release Linux                                          */
/* Version 1.3   First release HP-UX                                          */
/******************************************************************************/
/*                                                                            */
/* Note: This plugin node implementation is not necessarily portable          */
/* to all platforms supported by MQSeries Integrator Version 2.               */
/*                                                                            */
/******************************************************************************/
/* Includes */
#ifdef WIN32
#include <windows.h>
#endif
#include <malloc.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wchar.h>

#include "lber.h"
#include "ldap.h"
#include "LdapPlugIn.h"


/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */
/*  Acquire storage for attribute and link to the context                     */
/*                                                                            */
/******************************************************************************/
ATTRIBUTE_TBL_ENTRY *insertAttributeTableEntry(NODE_CONTEXT_ST *context, CciChar *nodeName, CciChar *attrName, int type)
{
   static wchar_t *functionName = L"insertAttributeTableEntry()";

   ATTRIBUTE_TBL_ENTRY *entry = NULL;
   int attrSize;

   /* Allocate storage for the attribute entry */
   entry = malloc(sizeof(ATTRIBUTE_TBL_ENTRY));

   if (entry) {
      entry->next = 0;
      entry->type = type;

      switch (entry->type) {
         case CNI_TYPE_INTEGER:
            attrSize = 4;
            break;
         case CNI_TYPE_STRING:
            attrSize = MAX_STRING_SIZE;
         default:
            /* Error: Invalid datatype */
            break;
      } /* switch (entry->type) */

      /* Acquire and clear storage for attribute value */
      entry->valptr = malloc(attrSize);

      if (entry->valptr) memset(entry->valptr, 0, attrSize);

      /* Store the attribute name */
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      CciCharCpy(entry->name, attrName);
#else
      wcscpy(entry->name, attrName);
#endif

      /* Link an existing previous element to this one */
      if (context->attrListPrevious) context->attrListPrevious->next = entry;
      else if ((context->attrListHead) == 0) context->attrListHead = entry;

      /* Save the pointer to the previous element */
      context->attrListPrevious = entry;
   }
   else {
         /* Error: Unable to allocate memory */
   }

   return(entry);

}


/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */ 
/*  Acquire storage for an input terminal entry and link it to the context    */
/*                                                                            */
/******************************************************************************/
TERMINAL_LIST_ENTRY *insertInputTerminalListEntry(NODE_CONTEXT_ST *context, CciChar *terminalName)
{
   static wchar_t *functionName = L"insertInputTerminalListEntry()";

   TERMINAL_LIST_ENTRY *entry = NULL;
   int rc;

   entry = malloc(sizeof(TERMINAL_LIST_ENTRY));

   if (entry) {
      /* This entry is the current end of the list */
      entry->next = 0;

    /* Store the terminal name */
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      CciCharCpy(entry->name, terminalName);
#else
      wcscpy(entry->name, terminalName);
#endif

      /* Create terminal and save its handle */
      entry->handle = cniCreateInputTerminal(&rc, context->nodeObject, terminalName);

      /* Link an existing previous element to this one */
      if (context->inputTerminalListPrevious)
	      context->inputTerminalListPrevious->next = entry;
      else if ((context->inputTerminalListHead) == 0)
	      context->inputTerminalListHead = entry;

      /* Save the pointer to the previous element */
      context->inputTerminalListPrevious = entry;

   } /* if (entry) */

  return(entry);
}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */ 
/*  Acquire storage for an output terminal and link it to the context         */
/*                                                                            */
/******************************************************************************/
TERMINAL_LIST_ENTRY *insertOutputTerminalListEntry(NODE_CONTEXT_ST *context, CciChar *terminalName)
{
   static wchar_t *functionName = L"insertOutputTerminalListEntry()";

   TERMINAL_LIST_ENTRY *entry = NULL;
   int rc;

   entry = malloc(sizeof(TERMINAL_LIST_ENTRY));

   if (entry) {
      /* This entry is the current end of the list */
      entry->next = 0;

      /* Store the terminal name */
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      CciCharCpy(entry->name, terminalName);
#else
      wcscpy(entry->name, terminalName);
#endif

      /* Create terminal and save its handle */
      entry->handle = cniCreateOutputTerminal(&rc, context->nodeObject, terminalName);

      /* Link an existing previous element to this one */
      if (context->outputTerminalListPrevious)
	      context->outputTerminalListPrevious->next = entry;
      else if ((context->outputTerminalListHead) == 0)
	      context->outputTerminalListHead = entry;

      /* Save the pointer to the previous element */
      context->outputTerminalListPrevious = entry;
   }

   return(entry);
}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */ 
/*  Return the terminal object for a named input terminal                     */
/*                                                                            */
/******************************************************************************/
void *getInputTerminalHandle(NODE_CONTEXT_ST *context, CciChar *terminalName)
{
   static wchar_t *functionName = L"getInputTerminalHandle()";

   TERMINAL_LIST_ENTRY *p = ((NODE_CONTEXT_ST *)context)->inputTerminalListHead;

  /* Scan the list for a matching terminal name */
  while (p != 0) {
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      if (CciCharCmp(p->name, terminalName) == 0) {
#else
      if (wcscmp(p->name, terminalName) == 0) {
#endif
         /* Match found; return it */
         return(p->handle);
      }
      p = p->next;
   }

   /* No match was found */
   return(0);
}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */ 
/*  Return the terminal object for a named output terminal                    */
/*                                                                            */
/******************************************************************************/
void *getOutputTerminalHandle(NODE_CONTEXT_ST *context, CciChar *terminalName)
{
   static wchar_t *functionName = L"getOutputTerminalHandle()";

   TERMINAL_LIST_ENTRY *p = ((NODE_CONTEXT_ST *)context)->outputTerminalListHead;

   /* Scan the list for a matching terminal name */
   while (p != 0) {
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      if (CciCharCmp(p->name, terminalName) == 0) {
#else
      if (wcscmp(p->name, terminalName) == 0) {
#endif
         /* Match found; return it */
         return(p->handle);
      }
      p = p->next;

   } /* while (p != 0) */

   /* No match was found */
   return(0);
}



/******************************************************************************/
/* Node Implementation Functions                                              */
/*                                                                            */
/* These are mandatory and MUST be implemented by the programmer.             */
/*                                                                            */
/* createNodeContext                                                          */
/* deleteNodeContext                                                          */
/* evaluate                                                                   */
/* getAttribute                                                               */
/* getAttriubteName                                                           */
/* setAttribute                                                               */
/* bipGetMessageflowNodeFactory                                               */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                cniCreateNodeContext() */
/*                                                                            */
/******************************************************************************/
CciContext *createNodeContext(void *factoryObject, CciChar *nodeName, void *nodeObject)
{
   static wchar_t* functionName=L"createNodeContext()";
   NODE_CONTEXT_ST* p;

   /* Allocate a pointer to the local context */
   p = malloc(sizeof(NODE_CONTEXT_ST));

   /* If malloc was successful */
   if (p) {
      /* Clear the context area */
      memset (p, 0, sizeof(NODE_CONTEXT_ST)); /* Clear the context area */

      /* Save our node object pointer in our context */
      p->nodeObject = nodeObject;

      /* Save our node name */
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      CciCharNCpy((CciChar*)&p->nodeName, nodeName, MAX_NODE_NAME_LEN);
#else
      wcsncpy((wchar_t*)&p->nodeName, nodeName, MAX_NODE_NAME_LEN);
#endif

      /* Verify the name of the node is supported by the factory */
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      if (CciCharCmp(nodeName, constPlugInNode) == 0) {
#else
      if (wcscmp(nodeName, L"LdapPlugInNode") == 0) {
#endif
          /* Create attributes */
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
         {
            const CciChar *ucsAttr = CciString("Server") ;
            insertAttributeTableEntry(p, nodeName, (CciChar*)ucsAttr, CNI_TYPE_STRING);
            free((void *)ucsAttr) ;
         }
         {
            const CciChar *ucsAttr = CciString("Port") ;
            insertAttributeTableEntry(p, nodeName, (CciChar*)ucsAttr, CNI_TYPE_STRING);
            free((void *)ucsAttr) ;
         }
         {
            const CciChar *ucsAttr = CciString("BindDN") ;
            insertAttributeTableEntry(p, nodeName, (CciChar*)ucsAttr, CNI_TYPE_STRING);
            free((void *)ucsAttr) ;
         }
         {
            const CciChar *ucsAttr = CciString("Password") ;
            insertAttributeTableEntry(p, nodeName, (CciChar*)ucsAttr, CNI_TYPE_STRING);
            free((void *)ucsAttr) ;
         }
         {
            const CciChar *ucsAttr = CciString("SearchbaseElement") ;
            insertAttributeTableEntry(p, nodeName, (CciChar*)ucsAttr, CNI_TYPE_STRING);
            free((void *)ucsAttr) ;
         }
         {
            const CciChar *ucsAttr = CciString("FilterElement") ;
            insertAttributeTableEntry(p, nodeName, (CciChar*)ucsAttr, CNI_TYPE_STRING);
            free((void *)ucsAttr) ;
         }
         {
            const CciChar *ucsAttr = CciString("SearchResultsRoot") ;
            insertAttributeTableEntry(p, nodeName, (CciChar*)ucsAttr, CNI_TYPE_STRING);
            free((void *)ucsAttr) ;
         }
         {
            const CciChar *ucsAttr = CciString("Searchbase") ;
            insertAttributeTableEntry(p, nodeName, (CciChar*)ucsAttr, CNI_TYPE_STRING);
            free((void *)ucsAttr) ;
         }
         {
            const CciChar *ucsAttr = CciString("Filter") ;
            insertAttributeTableEntry(p, nodeName, (CciChar*)ucsAttr, CNI_TYPE_STRING);
            free((void *)ucsAttr) ;
         }
#else
          insertAttributeTableEntry(p, nodeName, L"Server"            , CNI_TYPE_STRING);
          insertAttributeTableEntry(p, nodeName, L"Port"              , CNI_TYPE_STRING);
          insertAttributeTableEntry(p, nodeName, L"BindDN"            , CNI_TYPE_STRING);
          insertAttributeTableEntry(p, nodeName, L"Password"          , CNI_TYPE_STRING);
          insertAttributeTableEntry(p, nodeName, L"SearchbaseElement" , CNI_TYPE_STRING);
          insertAttributeTableEntry(p, nodeName, L"FilterElement"     , CNI_TYPE_STRING);
          insertAttributeTableEntry(p, nodeName, L"SearchResultsRoot" , CNI_TYPE_STRING);
          insertAttributeTableEntry(p, nodeName, L"Searchbase"        , CNI_TYPE_STRING);
          insertAttributeTableEntry(p, nodeName, L"Filter"            , CNI_TYPE_STRING);
#endif
          /* Create terminals */
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
         {
            const CciChar* ucsIn = CciString("in") ;
            insertInputTerminalListEntry(p, (CciChar*)ucsIn);
            free((void *)ucsIn) ;
         }
    
         {
            const CciChar* ucsOut = CciString("out") ;
            insertOutputTerminalListEntry(p, (CciChar*)ucsOut);
            free((void *)ucsOut) ;
         }

         {
            const CciChar* ucsFailure = CciString("failure") ;
            insertOutputTerminalListEntry(p, (CciChar*)ucsFailure);
            free((void *)ucsFailure) ;
         }
#else
          insertInputTerminalListEntry(p, L"in");
          insertOutputTerminalListEntry(p, L"out");
          insertOutputTerminalListEntry(p, L"failure");
#endif
      }
      else{
         /* Error handling if name of node is not supported by factory */
      }
   }
   else {
      /* Error handling if malloc failed */
   }

   /* Return the pointer to the local context */
   return(p);
}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                     cniGetAttribute() */
/*                                                                            */
/******************************************************************************/
int getAttribute(CciContext *context, CciChar *attrName, CciChar *buffer, int bufsize)
{
   static wchar_t *functionName = L"getAttribute()";

   ATTRIBUTE_TBL_ENTRY *p = ((NODE_CONTEXT_ST *)context)->attrListHead;
   NODE_CONTEXT_ST     *nc = context;

   wchar_t wstring[256];
   char    string[256];  

   while (p != 0) {
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      if (CciCharCmp(p->name, attrName) == 0) {
#else
      if (wcscmp(p->name, attrName) == 0) {
#endif
         switch(p->type) {
            case CNI_TYPE_INTEGER:
               sprintf((char*)&string, "%d", *((int*)p->valptr));
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
               {
                  const CciChar* ucsAttrValue = CciString(string);
                  CciCharNCpy(buffer, ucsAttrValue, min(bufsize, 256));
                  free((void *)ucsAttrValue);
               }
#else
               mbstowcs((wchar_t*)&wstring, (char*)&string, 256); 
               wcsncpy(buffer, (wchar_t*)&wstring, min(bufsize, 256));
#endif
               break;
            case CNI_TYPE_STRING:
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
               {
                  const CciChar* ucsAttrValue = CciString((char *)p->valptr);
                  CciCharNCpy(buffer, ucsAttrValue, min(MAX_STRING_SIZE, bufsize));
                  free((void *)ucsAttrValue);
               }
#else
               memcpy(buffer, p->valptr, min(MAX_STRING_SIZE, bufsize));
#endif
            default:
            /* Error: Invalid datatype */
            break;
         }

         /* Normal return; attribute value returned */ 
         return(0);
      }

      p = p->next;
   }

   /* Attribute not owned by plugin */
   return(1); 
}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                     cniSetAttribute() */
/*                                                                            */
/******************************************************************************/
int setAttribute(CciContext *context, CciChar *attrName, CciChar *attrValue)
{
   static wchar_t      *functionName = L"setAttribute()";

   ATTRIBUTE_TBL_ENTRY *p            = ((NODE_CONTEXT_ST *)context)->attrListHead;
   
   while (p != 0) {
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      if (CciCharCmp(p->name, attrName) == 0) {
#else
      if (wcscmp(p->name, attrName) == 0) {
#endif
         switch(p->type) {
            case CNI_TYPE_INTEGER:
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
               {
                  const char *mbAttrValue = mbString(attrValue);
                  *((int *)p->valptr) = atoi(mbAttrValue);
                  free((void *)mbAttrValue);
               }
#elif defined(__CCI_AIX__)
               *((int *)p->valptr) = watoi(attrValue);
#else
               *((int *)p->valptr) = _wtoi(attrValue);
#endif
               break;
            case CNI_TYPE_STRING:
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
               {
                  const char *mbAttrValue = mbString(attrValue);
                  memcpy(p->valptr, mbAttrValue, MAX_STRING_SIZE);
                  free((void *)mbAttrValue);
               }
#else
               memcpy(p->valptr, attrValue, MAX_STRING_SIZE); 
#endif
            default:
               /* Error: Invalid datatype */
               break;
         }

         /* Normal return; attribute value stored by plugin */
         return(0);
      }

      p = p->next;
   }

   /* Attribute not owned by plugin */
   return(1);
}


/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                     cniGetAttribute() */
/*                                                                            */
/******************************************************************************/
int getAttributeName(CciContext* context, int index, CciChar* buffer, int bufferSize)
{
   static wchar_t      *functionName = L"getAttributeName()";

   ATTRIBUTE_TBL_ENTRY *p            = ((NODE_CONTEXT_ST *)context)->attrListHead;
   int count = 0;
    
   while (p != 0) {
      if (count == index) {
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
         CciCharCpy(buffer, p->name);
#else
         wcscpy(buffer, p->name);
#endif
         /* Normal return; attribute name returned */
         return(0);
      }
      count++;
      p = p->next;
   }

   /* Attribute not owned by the plugin node; defer to base */
   return(1);
}



/**************************************************************************************************/
/* evaluate                                                                                       */
/* Description:                                                                                   */
/* Performs node processing. This function contains the main processing logic                     */
/* of the message flow node. It is invoked by the broker when a message is                        */
/* received on one of the input terminals of an instance of a node object.                        */
/**************************************************************************************************/
void evaluate(CciContext *context, CciMessage *destinationList, CciMessage *exceptionList, CciMessage *message)
{
   static wchar_t* functionName=L"evaluate()";
   CCI_EXCEPTION_ST exception_st;
      
   int              rc       = 0;
   int              retvalue = 1;
   int              Error    = 0;
   int              ctr;
   int              i;
   char             *ptr1;
   char             *ptr2;
   int              StrLen;
   int              Found;
   char             NextElementName[256];

   LDAP             *hLdap;
   LDAPMessage      *LdapEntry;
   BerElement       *LdapBer;
   char             *LdapAttr;
   char             **LdapValues;
   LDAPMessage      *LdapResult;

   char             *LdapServer            = NULL;
   char             *LdapPort              = NULL;
   char             *LdapBindDN            = NULL;
   char             *LdapPassword          = NULL;
   char             *LdapSearchbaseElement = NULL;
   char             *LdapFilterElement     = NULL;
   char             *LdapSearchResultsRoot = NULL;
   char             *LdapSearchbase        = NULL;
   char             *LdapFilter            = NULL;
   CciChar          *wLdapAttr             = NULL;
   CciChar          *wLdapValue            = NULL;
   CciChar          AttributeBuffer[256];
   CciChar          *wNextElementName      = NULL;

#ifdef DEBUG
   FILE             *outputFile = NULL;
#endif
      
   CciTerminal      *outputTerminalObject;
   CciElement       *Element;
   CciElement       *NextElement;
   CciElement       *InRootElement;
   CciElement       *OutRootElement;
   CciElement       *EntryElement;
   CciElement       *AttrElement;
   CciElement       *ValueNameElement;
   CciElement       *ValueElement;
   CciMessage       *outMsg = NULL;
   CciChar          elementName[256];
   CciChar          valueChar[256];
   CciSize          elementNameSize;

#ifdef DEBUG
#if defined(__CCI_LINUX__)																		/* 16/09/2001 */
   outputFile   = fopen("/tmp/LdapPlugInDebug.txt", "a");
   fprintf(outputFile, "\n\n*******************************************************************\n");
   fprintf(outputFile,     "*  Node \"LdapPlugIn\" for MQSeries Integrator V2.0 for Linux     *\n");
   fprintf(outputFile,     "*******************************************************************\n");
#elif defined(__CCI_SOLARIS__)
   outputFile   = fopen("/tmp/LdapPlugInDebug.txt", "a");
   fprintf(outputFile, "\n\n*******************************************************************\n");
   fprintf(outputFile,     "*  Node \"LdapPlugIn\" for MQSeries Integrator V2.0 for Solaris     *\n");
   fprintf(outputFile,     "*******************************************************************\n");
#elif defined(__CCI_HPUX__)																	/* 22/07/2001 */
   outputFile   = fopen("/tmp/LdapPlugInDebug.txt", "a");
   fprintf(outputFile, "\n\n*******************************************************************\n");
   fprintf(outputFile,     "*  Node \"LdapPlugIn\" for MQSeries Integrator V2.0 for HP-UX        *\n");
   fprintf(outputFile,     "*******************************************************************\n");
#elif defined(__CCI_AIX__)
   outputFile   = fopen("/tmp/LdapPlugInDebug.txt", "a");
   fprintf(outputFile, "\n\n*******************************************************************\n");
   fprintf(outputFile,     "*  Node \"LdapPlugIn\" for MQSeries Integrator V2.0 for AIX         *\n");
   fprintf(outputFile,     "*******************************************************************\n");
#else
   outputFile   = fopen("c:\\temp\\LdapPlugInDebug.txt", "a");
   fprintf(outputFile, "\n\n*******************************************************************\n");
   fprintf(outputFile,     "*  Node \"LdapPlugIn\" for MQSeries Integrator V2.0 for Windows NT  *\n");
   fprintf(outputFile,     "*******************************************************************\n");
#endif
   fflush(outputFile);
#endif
 
   /***********************************************************************************************/
   /* Get attribute values and transform strings to ascii.                                        */
   /***********************************************************************************************/
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      {																								/* 22/07/2001 */
         const CciChar *ucsAttr = CciString("Server") ;								/* 22/07/2001 */
         rc = getAttribute(context,(CciChar *)ucsAttr, AttributeBuffer, sizeof(AttributeBuffer));/* 22/07/2001 */
         free((void *)ucsAttr) ;																/* 22/07/2001 */
      }																								/* 22/07/2001 */
#else
   rc = getAttribute(context, L"Server"            , AttributeBuffer, sizeof(AttributeBuffer));
#endif
   LdapServer            = UnicodeToASCII(AttributeBuffer);
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      {																								/* 22/07/2001 */
         const CciChar *ucsAttr = CciString("Port");									/* 22/07/2001 */
         rc = getAttribute(context,(CciChar *)ucsAttr, AttributeBuffer, sizeof(AttributeBuffer));/* 22/07/2001 */
         free((void *)ucsAttr) ;																/* 22/07/2001 */
      }																								/* 22/07/2001 */
#else
   rc = getAttribute(context, L"Port"              , AttributeBuffer, sizeof(AttributeBuffer));
#endif
   LdapPort              = UnicodeToASCII(AttributeBuffer);
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      {																								/* 22/07/2001 */
         const CciChar *ucsAttr = CciString("BindDN") ;								/* 22/07/2001 */
         rc = getAttribute(context,(CciChar *)ucsAttr, AttributeBuffer, sizeof(AttributeBuffer));/* 22/07/2001 */
         free((void *)ucsAttr) ;																/* 22/07/2001 */
      }																								/* 22/07/2001 */
#else
   rc = getAttribute(context, L"BindDN"            , AttributeBuffer, sizeof(AttributeBuffer));
#endif
   LdapBindDN            = UnicodeToASCII(AttributeBuffer);
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      {																								/* 22/07/2001 */
         const CciChar *ucsAttr = CciString("Password");								/* 22/07/2001 */
         rc = getAttribute(context,(CciChar *)ucsAttr, AttributeBuffer, sizeof(AttributeBuffer));/* 22/07/2001 */
         free((void *)ucsAttr) ;																/* 22/07/2001 */
      }																								/* 22/07/2001 */
#else
   rc = getAttribute(context, L"Password"          , AttributeBuffer, sizeof(AttributeBuffer));
#endif
   LdapPassword          = UnicodeToASCII(AttributeBuffer);
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      {																								/* 22/07/2001 */
         const CciChar *ucsAttr = CciString("SearchbaseElement");					/* 22/07/2001 */
         rc = getAttribute(context,(CciChar *)ucsAttr, AttributeBuffer, sizeof(AttributeBuffer));/* 22/07/2001 */
         free((void *)ucsAttr) ;																/* 22/07/2001 */
      }																								/* 22/07/2001 */
#else
   rc = getAttribute(context, L"SearchbaseElement" , AttributeBuffer, sizeof(AttributeBuffer));
#endif
   LdapSearchbaseElement = UnicodeToASCII(AttributeBuffer);
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      {																								/* 22/07/2001 */
         const CciChar *ucsAttr = CciString("FilterElement");						/* 22/07/2001 */
         rc = getAttribute(context,(CciChar *)ucsAttr, AttributeBuffer, sizeof(AttributeBuffer));/* 22/07/2001 */
         free((void *)ucsAttr) ;																/* 22/07/2001 */
      }																								/* 22/07/2001 */
#else
   rc = getAttribute(context, L"FilterElement"     , AttributeBuffer, sizeof(AttributeBuffer));
#endif
   LdapFilterElement     = UnicodeToASCII(AttributeBuffer);
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      {																								/* 22/07/2001 */
         const CciChar *ucsAttr = CciString("SearchResultsRoot");					/* 22/07/2001 */
         rc = getAttribute(context,(CciChar *)ucsAttr, AttributeBuffer, sizeof(AttributeBuffer));/* 22/07/2001 */
         free((void *)ucsAttr) ;																/* 22/07/2001 */
      }																								/* 22/07/2001 */
#else
   rc = getAttribute(context, L"SearchResultsRoot" , AttributeBuffer, sizeof(AttributeBuffer));
#endif
   LdapSearchResultsRoot = UnicodeToASCII(AttributeBuffer);
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      {																								/* 22/07/2001 */
         const CciChar *ucsAttr = CciString("Searchbase");							/* 22/07/2001 */
         rc = getAttribute(context,(CciChar *)ucsAttr, AttributeBuffer, sizeof(AttributeBuffer));/* 22/07/2001 */
         free((void *)ucsAttr) ;																/* 22/07/2001 */
      }																								/* 22/07/2001 */
#else
   rc = getAttribute(context, L"Searchbase"        , AttributeBuffer, sizeof(AttributeBuffer));
#endif
   LdapSearchbase        = UnicodeToASCII(AttributeBuffer);
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      {																								/* 22/07/2001 */
         const CciChar *ucsAttr = CciString("Filter") ;								/* 22/07/2001 */
         rc = getAttribute(context,(CciChar *)ucsAttr, AttributeBuffer, sizeof(AttributeBuffer));/* 22/07/2001 */
         free((void *)ucsAttr) ;																/* 22/07/2001 */
      }																								/* 22/07/2001 */
#else
   rc = getAttribute(context, L"Filter"            , AttributeBuffer, sizeof(AttributeBuffer));
#endif
   LdapFilter            = UnicodeToASCII(AttributeBuffer);

#ifdef DEBUG
   fprintf(outputFile, "Node attributes:\n");
   fprintf(outputFile, "   LdapServer            = \"%s\"\n", LdapServer);
   fprintf(outputFile, "   LdapPort              = \"%s\"\n", LdapPort);
   fprintf(outputFile, "   LdapBindDN            = \"%s\"\n", LdapBindDN);
   fprintf(outputFile, "   LdapPassword          = \"%s\"\n", LdapPassword);
   fprintf(outputFile, "   LdapSearchbaseElement = \"%s\"\n", LdapSearchbaseElement);
   fprintf(outputFile, "   LdapFilterElement     = \"%s\"\n", LdapFilterElement);
   fprintf(outputFile, "   LdapSearchResultsRoot = \"%s\"\n", LdapSearchResultsRoot);
   fprintf(outputFile, "   LdapSearchbase        = \"%s\"\n", LdapSearchbase);
   fprintf(outputFile, "   LdapFilter            = \"%s\"\n", LdapFilter);
   fflush(outputFile);
#endif

   /***********************************************************************************************/
   /* Determine LDAP searchbase:                                                                  */
   /*    - if the element LdapSearchbaseElement exists in the tree, then use this value           */
   /*      to overwrite the value of LdapSearchbase;                                              */
   /*    - otherwise, use LdapSearchbase straightaway.                                            */
   /***********************************************************************************************/

#ifdef DEBUG
   fprintf(outputFile, "Start navigating to LdapSearchbaseElement\n");
   fflush(outputFile);
#endif

   /* Get the root element of the message */
   InRootElement = cniRootElement(&rc, message);
   Element = InRootElement;
   Found = 0;
   ptr1 = strstr(LdapSearchbaseElement, ".");

   while (ptr1) {
      ptr1++;
      ptr2 = strstr(ptr1, ".");
      if (ptr2)
         StrLen = (int) (ptr2-ptr1);
      else
         StrLen = strlen(LdapSearchbaseElement) - (int) (ptr1 - LdapSearchbaseElement);
      memcpy(NextElementName, ptr1, StrLen);
      NextElementName[StrLen] = '\0';
      wNextElementName = ASCIIToUnicode(NextElementName);
#ifdef DEBUG
      fprintf(outputFile, "   NextElementName = %s\n", NextElementName);
      fflush(outputFile);
#endif

      /* Go to the first child of this element */
      NextElement = cniFirstChild(&rc, Element);

      /* Loop until element found or no elements left */
      Found = 0;

      while (NextElement) {
         /* Get the element's name */
         elementNameSize = cniElementName(&rc, NextElement, (CciChar*)&elementName, sizeof(elementName));
         /* Check if name is right */
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
			if (CciCharCmp(elementName, wNextElementName) == 0) {						/* 22/07/2001 */
#else																									/* 22/07/2001 */
         if ( wcscmp(elementName, wNextElementName) == 0 ) {
#endif																								/* 22/07/2001 */
            Found = 1;
            break;
         }
         NextElement = cniNextSibling(&rc, NextElement);
      }

      if (Found == 1) {
         /* Element found: go to this child */
         Element = NextElement;
      }
      else {
         /* Element not found: break from loop */
#ifdef DEBUG
         fprintf(outputFile, "   Path does not exist, using original value of LdapSearchbase\n");
         fflush(outputFile);
#endif
         Found = 0;
         break;
      }

      ptr1 = ptr2;
   }

   if (Found == 1) {
      /* LdapSearchbaseElement path exists: read LDAP searchbase value from this element */
      cniElementCharacterValue(&rc, Element, (CciChar *)&valueChar, sizeof(valueChar));
      free(LdapSearchbase);
      LdapSearchbase = UnicodeToASCII(valueChar);
#ifdef DEBUG
      fprintf(outputFile, "   Updated LdapSearchbase = \"%s\"\n", LdapSearchbase);
      fflush(outputFile);
#endif
   }

   /***********************************************************************************************/
   /* Determine LDAP filter:                                                                      */
   /*    - if the element LdapFilterElement exists in the tree, then use this value               */
   /*      to overwrite the value of LdapFilter;                                                  */
   /*    - otherwise, use LdapFilter straightaway.                                                */
   /***********************************************************************************************/
#ifdef DEBUG
   fprintf(outputFile, "Start navigating to LdapFilterElement\n");
   fflush(outputFile);
#endif

   /* Get the root element of the message */
   InRootElement = cniRootElement(&rc, message);
   Element = InRootElement;
   Found = 0;
   ptr1 = strstr(LdapFilterElement, ".");

   while (ptr1) {
      ptr1++;
      ptr2 = strstr(ptr1, ".");
      if (ptr2)
         StrLen = (int) (ptr2-ptr1);
      else
         StrLen = strlen(LdapFilterElement) - (int) (ptr1 - LdapFilterElement);
      memcpy(NextElementName, ptr1, StrLen);
      NextElementName[StrLen] = '\0';
      wNextElementName = ASCIIToUnicode(NextElementName);
#ifdef DEBUG
      fprintf(outputFile, "   NextElementName = %s\n", NextElementName);
      fflush(outputFile);
#endif

      /* Go to the first child of this element */
      NextElement = cniFirstChild(&rc, Element);

      /* Loop until element found or no elements left */
      Found = 0;

      while (NextElement) {
         /* Get the element's name */
         elementNameSize = cniElementName(&rc, NextElement, (CciChar*)&elementName, sizeof(elementName));
         /* Check if name is right */
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
			if (CciCharCmp(elementName, wNextElementName) == 0) {						/* 22/07/2001 */
#else																									/* 22/07/2001 */
         if ( wcscmp(elementName, wNextElementName) == 0 ) {
#endif
            Found = 1;
            break;
         }
         NextElement = cniNextSibling(&rc, NextElement);
      }

      if (Found == 1) {
         /* Element found: go to this child */
         Element = NextElement;
      }
      else {
         /* Element not found: break from loop */
#ifdef DEBUG
         fprintf(outputFile, "   Path does not exist, using original value of LdapFilter\n");
         fflush(outputFile);
#endif
         Found = 0;
         break;
      }
      ptr1 = ptr2;
   }

   if (Found == 1) {
      /* LdapFilterElement path exists: read LDAP filter value from this element */
      cniElementCharacterValue(&rc, Element, (CciChar*)&valueChar, sizeof(valueChar));
      free(LdapFilter);
      LdapFilter = UnicodeToASCII(valueChar);
#ifdef DEBUG
      fprintf(outputFile, "   Updated LdapFilter = \"%s\"\n", LdapFilter);
      fflush(outputFile);
#endif
   }

   /***********************************************************************************************/
   /* ldap_open: get a handle to an LDAP connection using LdapServer and LdapPort.                */
   /***********************************************************************************************/

   hLdap = ldap_open( LdapServer,
                      atoi(LdapPort) );

   if ( hLdap == NULL ) {
      Error = 1;
#ifdef DEBUG
      fprintf(outputFile, "ERROR: ldap_init failed!\n");
      fflush(outputFile);
#endif
   }

   /***********************************************************************************************/
   /* ldap_simple_bind_s: authenticate to the directory using LdapBindDN and LdapPassword.        */
   /***********************************************************************************************/

   if (Error == 0) {
      rc = ldap_simple_bind_s( hLdap,
                               LdapBindDN,
                               LdapPassword );

      if (rc != LDAP_SUCCESS) {
         Error = 1;
#ifdef DEBUG
         fprintf(outputFile, "ERROR: ldap_simple_bind_s failed with rc=%d!\n", rc);
         fflush(outputFile);
#endif
      }
   }

   /***********************************************************************************************/
   /* ldap_search_s: search the directory using LdapSearchbase and LdapFilter.                    */
   /***********************************************************************************************/

   if (Error == 0) {
      rc = ldap_search_s( hLdap,
                          LdapSearchbase,
                          LDAP_SCOPE_SUBTREE,
                          LdapFilter,
                          NULL,
                          0,
                          &LdapResult );

      if ( rc != LDAP_SUCCESS ) {
         Error = 1;
#ifdef DEBUG
         fprintf(outputFile, "ERROR: ldap_search_ext_s failed!\n");
         fflush(outputFile);
#endif
      }
   }

   /***********************************************************************************************/
   /* Create a new output message as a copy of the input message.                                 */
   /***********************************************************************************************/

   /* Get the root element of the message */
   InRootElement = cniRootElement(&rc, message);

   /* Create a new message for output */
   outMsg = cniCreateMessage(&rc, cniGetMessageContext(&rc, message));
   CCI_CHECK_RC();

   /* Get the root element of the output message */
   OutRootElement = cniRootElement(&rc, outMsg);
   CCI_CHECK_RC();

   /* Copy the contents of the input message to the output message */
   cniCopyElementTree(&rc, InRootElement, OutRootElement);
   CCI_CHECK_RC();

   /***********************************************************************************************/
   /* Navigate to the LdapSearchResultsRoot in the output message.                                */
   /* If this path does not exists: create it.                                                    */
   /***********************************************************************************************/

#ifdef DEBUG
   fprintf(outputFile, "Start navigating to LdapSearchResultsRoot\n");
   fflush(outputFile);
#endif
   Element = OutRootElement;
   ptr1 = strstr(LdapSearchResultsRoot, ".");
   while (ptr1) {
      ptr1++;
      ptr2 = strstr(ptr1, ".");
      if (ptr2)
         StrLen = (int) (ptr2-ptr1);
      else
         StrLen = strlen(LdapSearchResultsRoot) - (int) (ptr1 - LdapSearchResultsRoot);
      memcpy(NextElementName, ptr1, StrLen);
      NextElementName[StrLen] = '\0';
      wNextElementName = ASCIIToUnicode(NextElementName);
#ifdef DEBUG
      fprintf(outputFile, "   NextElementName = %s\n", NextElementName);
      fflush(outputFile);
#endif

      /* Go to the first child of this element */
      NextElement = cniFirstChild(&rc, Element);

      /* Loop until element found or no elements left */
      Found = 0;

      while (NextElement) {
         /* Get the element's name */
         elementNameSize = cniElementName(&rc, NextElement, (CciChar*)&elementName, sizeof(elementName));
         /* Check if name is right */
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
			if (CciCharCmp(elementName, wNextElementName) == 0) {						/* 22/07/2001 */
#else																									/* 22/07/2001 */
         if ( wcscmp(elementName, wNextElementName) == 0 ) {
#endif
            Found = 1;
            break;
         }
         NextElement = cniNextSibling(&rc, NextElement);
      }

      if (Found == 1) {
         /* Element found: go to this child */
         Element = NextElement;
      }
      else {
         /* Element not found: create new child */
         Element = cniCreateElementAsLastChild(&rc, Element);
         cniSetElementName(&rc, Element, wNextElementName);
         cniSetElementType(&rc, Element, CCI_ELEMENT_TYPE_NAME);
      }
      ptr1 = ptr2;
   }

   /***********************************************************************************************/
   /* Copy the LDAP search results into the message tree.                                         */
   /***********************************************************************************************/

   if (Error == 0) {
      for ( LdapEntry = ldap_first_entry( hLdap, LdapResult );
            LdapEntry != NULL;
            LdapEntry = ldap_next_entry( hLdap, LdapEntry ) )
      {
#ifdef DEBUG
         fprintf(outputFile, "Entry:\n");
         fflush(outputFile);
#endif

         /* Add a tag "Entry" to the message */
         EntryElement = cniCreateElementAsLastChild(&rc, Element);
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
			cniSetElementName(&rc, EntryElement, (CciChar *)constEntry);			/* 22/07/2001 */
#else																									/* 22/07/2001 */
         cniSetElementName(&rc, EntryElement, L"Entry");
#endif
         cniSetElementType(&rc, EntryElement, CCI_ELEMENT_TYPE_NAME);

         for ( LdapAttr = ldap_first_attribute( hLdap, LdapEntry, &LdapBer );
               LdapAttr != NULL;
               LdapAttr = ldap_next_attribute( hLdap, LdapEntry, LdapBer ) )
         {
#ifdef DEBUG
            fprintf(outputFile, "   AttributeName=<%s>\n", LdapAttr);
            fflush(outputFile);
#endif

            /* Add a tag with attribute name to the message */
            AttrElement = cniCreateElementAsLastChild(&rc, EntryElement);
            wLdapAttr = ASCIIToUnicode(LdapAttr);
            cniSetElementName(&rc, AttrElement, wLdapAttr);
            cniSetElementType(&rc, AttrElement, CCI_ELEMENT_TYPE_NAME);

            if ((LdapValues = ldap_get_values( hLdap, LdapEntry, LdapAttr)) != NULL ) {
               for ( i = 0; LdapValues[i] != NULL; i++ )
               {
#ifdef DEBUG
                  fprintf(outputFile, "      AttributeValue=<%s>\n", LdapValues[i]);
                  fflush(outputFile);
#endif

                  /* Add a tag "Value" to the message */
                  ValueNameElement = cniCreateElementAsLastChild(&rc, AttrElement);
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
						cniSetElementName(&rc, ValueNameElement, (CciChar *)constValue);/* 22/07/2001 */
#else																									/* 22/07/2001 */
                  cniSetElementName(&rc, ValueNameElement, L"Value");
#endif
                  cniSetElementType(&rc, ValueNameElement, CCI_ELEMENT_TYPE_NAME);

                  /* Add the value to the message */
                  ValueElement = cniCreateElementAsLastChild(&rc, ValueNameElement);
                  cniSetElementType(&rc, ValueElement, CCI_ELEMENT_TYPE_VALUE);
                  wLdapValue = ASCIIToUnicode(LdapValues[i]);
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
                  cniSetElementCharacterValue(&rc, ValueElement, (const CciChar *)wLdapValue,
                                              CciCharLen(wLdapValue));
#else
                  cniSetElementCharacterValue(&rc, ValueElement, (const CciChar *)wLdapValue,
                                              wcslen(wLdapValue));
#endif
               }
               ldap_value_free( LdapValues );
            }
            ldap_memfree( LdapAttr );
         }
         if ( LdapBer != NULL ) {
            ber_free( LdapBer, 0 );
         }
         ctr++;
      }
   }
   ldap_msgfree( LdapResult );
   ldap_unbind( hLdap );

   /***********************************************************************************************/
   /* Evaluate exit                                                                               */
   /***********************************************************************************************/

   cniWriteBuffer(&rc, outMsg);

   if (Error == 0) {
#ifdef DEBUG
      fprintf(outputFile, "Propagating to terminal \"out\"\n");
      fflush(outputFile);
#endif
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      outputTerminalObject = getOutputTerminalHandle(context, (CciChar*)constOut);/* 22/07/2001 */
#else																									/* 22/07/2001 */
      outputTerminalObject = getOutputTerminalHandle(context, L"out");   
#endif
   }
   else {
#ifdef DEBUG
      fprintf(outputFile, "Propagating to terminal \"failure\"\n");
      fflush(outputFile);
#endif
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      outputTerminalObject = getOutputTerminalHandle(context, (CciChar*)constFailure);/* 22/07/2001 */
#else																									/* 22/07/2001 */
      outputTerminalObject = getOutputTerminalHandle(context, L"failure");   
#endif
   }

   message = outMsg;

   /***********************************************************************************************/
   /* Free allocated memory.                                                                      */
   /***********************************************************************************************/

   free(wNextElementName     );
   free(LdapServer           );
   free(LdapPort             );
   free(LdapBindDN           );
   free(LdapPassword         );
   free(LdapSearchbaseElement);
   free(LdapFilterElement    );
   free(LdapSearchResultsRoot);
   free(LdapSearchbase       );
   free(LdapFilter           );
   free(wLdapAttr            );
   free(wLdapValue           );
  
#ifdef DEBUG
   fclose(outputFile);
#endif

   /* If the terminal exists and is attached, propagate to it */
   if (outputTerminalObject) {
      if (cniIsTerminalAttached(&rc, outputTerminalObject)) {
         if (rc == CCI_SUCCESS) {
            retvalue = cniPropagate(&rc, outputTerminalObject, destinationList, exceptionList, message);
            if (rc != CCI_SUCCESS) {
               if (rc == CCI_EXCEPTION) {
                  /* Get details of the exception */
                  memset(&exception_st, 0, sizeof(exception_st));
                  cciGetLastExceptionData(&rc, &exception_st);
                  /* Any local error handling may go here */
                  /* If the exception is "caught" it must be re-thrown */
                  cciRethrowLastException(0);
               }
               /* Some other error...log it */
               cciLog(&rc, CCI_LOG_ERROR, __FILE__, __LINE__, "",
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
                 (CciChar*)constMQSeriesIntegrator2, 0, "Propagate failed", 0);
#else
                 L"MQSIv201", 0, "Propagate failed", 0);
#endif
            } /* if (rc != CCI_SUCCESS) */
            else {
               /* Message was propagated successfully */
               cciLog(&rc, CCI_LOG_INFORMATION, __FILE__, __LINE__, "",
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
                 (CciChar*)constMQSeriesIntegrator2, 0, "Message propagated", 0);
#else 
                 L"MQSIv201", 0, "Message propagated", 0);
#endif 
            }
         } /* if (rc == CCI_SUCCESS) */
         else {
            if (rc == CCI_EXCEPTION) {
               cciRethrowLastException(0);
            }
            else {
               /* Some other error...log it */
               cciLog(&rc, CCI_LOG_ERROR, __FILE__, __LINE__, "",
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
                 (CciChar*)constMQSeriesIntegrator2, 0, "Error occurred", 0);
#else 
                 L"MQSIv201", 0, "Error occurred", 0);
#endif 
            }
         }
      }
   }
   else {
      /* Terminal did not exist...severe internal error */
      cciLog(&rc, CCI_LOG_ERROR, __FILE__, __LINE__, "", 
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
           (CciChar*)constMQSeriesIntegrator2, 0, "Terminal not connected", 0);
#else 
           L"MQSIv201", 0, "Terminal not connected", 0);
#endif 
   }

   if (outMsg != NULL) {
      /* Delete the message we created now we have finished with it */
      cniDeleteMessage(0, outMsg);
   }

   return;
}


#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */
/*  initNodeConstants                                                         */
/*                                                                            */
/*  Creates Cci constant strings from defined char * constant strings.        */
/*                                                                            */
/******************************************************************************/
void initNodeConstants(void)
{
  constMQSeriesIntegrator2 = CciString(CONST_MQSERIES_INTEGRATOR_2);
  constPlugInNode          = CciString(CONST_PLUGIN_NODE);
  constPluginNodeFactory   = CciString(CONST_PLUGIN_NODE_FACTORY);
  constOut                 = CciString(CONST_OUT);
  constFailure             = CciString(CONST_FAILURE);

  constEntry					= CciString(CONST_ENTRY);
  constValue					= CciString(CONST_VALUE);
}
#endif

/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                                       */
/*                                                                            */
/* This function is called when the message broker loads the "lil" during the */
/* initialization of the broker engine.                                       */
/*                                                                            */
/* The responsibilities of the plug-in are to:                                */
/*  - create the node factory (or factories) which the "lil" supports, using  */
/*    the createNodeFactory() function, saving the returned pointer to the    */
/*    node factory for use at runtime.                                        */
/*  - define each node supported by the (or each) factory) and pass a pointer */
/*    to a function table which contains pointers to the node implementation  */
/*    functions.                                                              */
/*  - define the name of each attribute supported by each node, including a   */
/*    pointer to a validation function.                                       */
/*  - define input and output terminals supported by each node.               */
/*  - return the pointer to the factory, unless an error occurred, in which   */
/*    case zero should be returned.                                           */    
/*                                                                            */
/******************************************************************************/
CciFactory LilFactoryExportPrefix *LilFactoryExportSuffix *bipGetMessageflowNodeFactory()
{
   static wchar_t*   functionName=L"bipGetMessageflowNodeFactory()";
   CciFactory*       factoryObject;
   int               rc = 0;
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
#else
   CciChar           factoryName[] = L"LdapPlugInNodeFactory";
#endif
   CCI_EXCEPTION_ST  exception_st;
   static CNI_VFT    vftable;

#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
   /* Before we proceed we need to initialise all the static constants */
   /* that may be used by the plug-in.                                 */
   initNodeConstants();
#endif
   /* Create the Node Factory for this plug-in */
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
   factoryObject = cniCreateNodeFactory(0, (unsigned short *)constPluginNodeFactory);
#else
   factoryObject = cniCreateNodeFactory(0, factoryName);
#endif

   if (factoryObject == CCI_NULL_ADDR) {
      if (rc == CCI_EXCEPTION) {
         /* Get details of the exception */
         memset(&exception_st, 0, sizeof(exception_st));
         cciGetLastExceptionData(&rc, &exception_st);

         /* Any local error handling may go here */

         /* Rethrow the exception */
         cciRethrowLastException(&rc);
      }
      /* Any further local error handling can go here */
   }
   else {
      /* Setup function table with pointers to node implementation functions */
      memset(&vftable, 0, sizeof(vftable));
      vftable.iFpCreateNodeContext     = createNodeContext;
      vftable.iFpGetAttributeName      = getAttributeName;
      vftable.iFpSetAttribute          = setAttribute;
      vftable.iFpGetAttribute          = getAttribute;
      vftable.iFpEvaluate              = evaluate;

      /* Define a node type supported by our factory. If any errors/exceptions    */
      /* occur during the execution of this utility function, then as we have not */
      /* supplied the returnCode argument, the exception will bypass the plugin   */
      /* and be directly handled by the broker.                                   */
#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
      cniDefineNodeClass(0, factoryObject, (CciChar*)constPlugInNode, &vftable);
#else
      cniDefineNodeClass(&rc, factoryObject, L"LdapPlugInNode", &vftable); 
#endif

      if (rc != CCI_SUCCESS) {
         /* Error Handling */
      }
   }

   /* Return address of this factory object to the broker */
   return(factoryObject);
}



/* 22/07/2001 */
CciChar *ASCIIToUnicode(const char *aStr)
{
   unsigned int len = strlen(aStr);
	int rc;

   CciChar *outStr = (CciChar *)malloc(sizeof(CciChar) * (len + 1));
	cciMbsToUcs(&rc, aStr, outStr, len + 1, 1208) ;

   outStr[len] = 0;
   return outStr;
}


/* 22/07/2001 */
char *UnicodeToASCII(const CciChar *lStr)
{
	int rc;

#if defined(__CCI_SOLARIS__) || defined(__CCI_HPUX__)|| defined(__CCI_LINUX__)/* 16/09/2001 */
	unsigned int len = CciCharLen(lStr);
#else
	unsigned int len = wcslen(lStr);
#endif
   char *outStr = (char *)malloc(sizeof(char) * (len + 1));
   cciUcsToMbs(&rc, lStr, outStr, len + 1, 1208);

	outStr[len] = 0;
	return outStr;
}



char *StrUpper(char *string)
{
   int i = 0;
   while ( ( string[i] = (char)toupper(string[i]) ) != '\0' )
      i++;
   return(string);
}

